/**
 * 
 */
package bpiwowar.argparser;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;

import bpiwowar.argparser.checkers.ValueChecker;
import bpiwowar.argparser.handlers.Handler;


/**
 * Marks a field that should be accessible from the command line
 * To check integrity, use {@link ArgumentPostProcessor}
 * @author bpiwowar
 * @date 30/08/2007
 *
 */
@Retention(value = RetentionPolicy.RUNTIME)
public @interface Argument {
	/** List of options in the command line */
	@Deprecated
	String[] names() default {};

	/** How to use the option on the command line (each name will be prefixed by --) */
	String[] name() default {};

    /** The help string */
    String help() default "";
    
    /** The group the option belongs to*/
	String group() default "";
	
	/** Fully qualified name of the wrapper object */
	Class<? extends Handler> handler() default Handler.class;

	/** Should the default value be displayed ? */
	boolean displayDefault() default false;

	/** 
	 * The valid intervals of values
	 * In case of arrays, this can be a list of intervals
	 * Example: [0,1),(2,3]
	 * Note that intervals are converted by the handler 
	 * 
	 * @Deprecated Use specific annotations for type checkers
	 */
	@Deprecated()
	String [] intervals() default {};

	/**
	 * List of value checkers (see {@link ValueChecker})
	 */
	Class<? extends ValueChecker>[] checkers() default {};

	/**
	 * Is this option mandatory (false by default)?
	 */
	boolean [] required() default {};

	/**
	 * Is this option active by defaut (true by default)? 
	 */
	boolean [] activated() default {};
	
	/**
	 * The ID of the option in case one wants to refer to it
	 */
	String id() default "";

	/**
	 * Prefix for sub-options
	 */
	String prefix() default "";

}
