package bpiwowar.argparser;

import java.util.ArrayList;
import java.util.Collection;

import bpiwowar.argparser.handlers.Handler;

/**
 * An object that represent <em>one</em> option of the command line
 * 
 * @author Benjamin Piwowarski <benjamin@bpiwowar.net>
 */
class Option {
	/**
	 * The result holder
	 */
	Handler handler = null;

	/**
	 * Should this record be specified in the command line?
	 */
	boolean mandatory;
	
	/** 
	 * Mandatory status
	 */
	MandatoryStatus mandatoryStatus;

	/**
	 * Was in the command line
	 */
	boolean processed;

	/**
	 * The list of possible names for the option
	 */
	ArrayList<OptionName> optionNames = new ArrayList<OptionName>();

	/**
	 * Help message
	 */
	String helpMessage = null;

	/**
	 * Is this option active?
	 */
	private boolean activated = true;

	/**
	 * Groups to activate if the option is set
	 */
	ArrayList<String> toActivate = new ArrayList<String>();

	/**
	 * Creates a new option
	 * 
	 * @param nameList
	 *            The list of command line
	 * @param handler
	 *            The handler/holder
	 * @param mandatory
	 *            If the option is mandatory
	 * @param helpMsg
	 *            The help message
	 * @param valueDesc
	 */
	public Option(OptionName[] nameList, Handler handler, boolean mandatory,
			String helpMsg, String valueDesc) {
		for (OptionName optionName : nameList)
			this.optionNames.add(optionName);
		this.handler = handler;
		this.mandatory = mandatory;
		this.helpMessage = helpMsg;
	}

	/**
	 * Default constructor
	 */
	public Option() {
	}

	/**
	 * @param mandatory
	 *            the mandatory to set
	 */
	public void setMandatory(boolean mandatory) {
		this.mandatory = mandatory;
	}

	/**
	 * @return the mandatory
	 */
	public boolean isMandatory() {
		return mandatory;
	}

	/**
	 * @param processed
	 *            the wasProcessed to set
	 */
	public void setProcessed(boolean processed) {
		this.processed = processed;
	}

	/**
	 * @return the wasProcessed
	 */
	public boolean isProcessed() {
		return processed;
	}

	/**
	 * Get the different option names
	 * 
	 * @return
	 */
	public String getNames() {
		String optionInfo = "";

		for (OptionName ndesc : optionNames) {
			if (!optionInfo.equals(""))
				optionInfo += ",";

			optionInfo += ndesc.name;
		}
		return optionInfo;
	}

	public void setActive(boolean activated) {
		this.activated = activated;
	}

	public boolean isActivated() {
		return activated;
	}

	public void setActivators(Collection<String> list) {
		toActivate.addAll(list);
	}
}