package bpiwowar.argparser.checkers;

import java.io.File;

import org.apache.log4j.Logger;

public class IOChecker {
	final static private Logger logger = Logger.getLogger(IOChecker.class);
	static public class CreateableFile implements ValueChecker {
		public boolean check(Object object) {
			if (object instanceof File) {
				final File file = (File) object;
				return file.getParentFile() == null || file.getParentFile().isDirectory();
			}
			
			throw new UnsupportedOperationException(
					"Cannot check that an object of type " + object.getClass()
							+ " is a creatable file");
		}

		public String getDescription() {
			return "cannot be created";
		}

	}

	static public class ValidFile implements ValueChecker {

		public boolean check(Object object) {
			logger.debug("Checking if " + object + " is a valid file");
			if (object instanceof File)
				return ((File) object).isFile();
			
			throw new UnsupportedOperationException(
					"Cannot check that an object of type " + object.getClass()
							+ " is a valid file");
		}

		public String getDescription() {
			return "is not a valid file";
		}
	}

	static public class Readable implements ValueChecker {

		public boolean check(Object object) {
			if (object instanceof File)
				return ((File) object).canRead();
			
			throw new UnsupportedOperationException(
					"Cannot check that an object of type " + object.getClass()
							+ " is readable");
		}

		public String getDescription() {
			return "is not readable";
		}
	}


	static public class ValidDirectory implements ValueChecker {

		public boolean check(Object object) {
			if (object instanceof File)
				return ((File) object).isDirectory();

			throw new UnsupportedOperationException(
					"Cannot check that an object of type " + object.getClass()
							+ " is a valid directory");
		}

		public String getDescription() {
			return "is not a valid directory";
		}

	}

}
