/**
 * 
 */
package bpiwowar.argparser.checkers;

/**
 * @author bpiwowar
 * @date Nov 29, 2007
 *
 */
public abstract class Limit implements ValueChecker, Comparable<Object> {
	public static enum Type {
		OPEN_MINIMUM(">"),
		CLOSED_MINIMUM(">="),

		OPEN_MAXIMUM("<"),
		CLOSED_MAXIMUM("<="),
		
		EQUALITY("="),
		
		UNKNOWN("?");
		
		private String description;
		Type(final String description) {
			this.description = description;
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Enum#toString()
		 */
		@Override
		public String toString() {
			return description;
		}
	}
	
	private Type type;
	
	public Limit(Type type) {
		this.type = type;
	}
	
	/* (non-Javadoc)
	 * @see yrla.argparser.checkers.ValueChecker#check()
	 */
	public boolean check(Object object) {
		int c = compareTo(object);
		switch (type) {
		case CLOSED_MAXIMUM:
			return c >= 0;
		case CLOSED_MINIMUM:
			return c <= 0;
			
		case OPEN_MAXIMUM:
			return c > 0;
		case OPEN_MINIMUM:
			return c < 0;
			
		case EQUALITY:
			return c == 0;
			
		case UNKNOWN:
			throw new IllegalStateException("Cannot check an unknown limit");
		}
		return false;
	}
	
	/**
	 * Sets the type of the limit
	 * @param type the type to set
	 */
	public void setType(Type type) {
		this.type = type;
	}

	/* (non-Javadoc)
	 * @see yrla.argparser.checkers.ValueChecker#getDescription()
	 */
	public String getDescription() {
		return type.toString();
	}
}
