/**
 * 
 */
package bpiwowar.argparser.handlers;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.lang.reflect.Field;

import bpiwowar.argparser.ArgParseException;
import bpiwowar.argparser.ArgParser;



/**
 * @author bpiwowar
 * @date 30/08/2007
 *
 */
@HandlerInformation({boolean.class, Boolean.class})
public class BooleanHandler extends GenericObjectsHandler {
	
	/**
	 * Used for a boolean-handled field when the boolean value
	 * must be specified
	 * @author bpiwowar
	 * @date Mar 19, 2008
	 */
	@Retention(RetentionPolicy.RUNTIME)
	@Target(ElementType.FIELD)
	public @interface NoDefaultValue {
	}
	
	Boolean defaultValue = null;

	/**
	 * Use this handler if the boolean value must be explicitely given
	 * @author bpiwowar
	 * @date Dec 5, 2007
	 */
	public static class NoSetValue extends BooleanHandler {
		public NoSetValue(Object object, Field field) {
			super(object, field);
			setDefault(null);
		}
	}
	
	public BooleanHandler(Object object, Field field) {
		super(object, field, Boolean.class);
		// If value set, take a default which is the inverse
		// of the value (Use NoSetValue)
		Object o = get();
		if (o != null && field.getAnnotation(NoDefaultValue.class) == null) {
			setDefault(!((Boolean) o).booleanValue());
		}
	}
	

	
	/* (non-Javadoc)
	 * @see argparser.BooleanHolderInterface#isValue()
	 */
	public boolean getValue() {
		try {
			return field.getBoolean(object);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/* (non-Javadoc)
	 * @see argparser.BooleanHolderInterface#setValue(boolean)
	 */
	public void setValue(boolean value) {
		try {
			field.setBoolean(object, value);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Set the default value
	 * @param b
	 */
	public void setDefault(Boolean b) {
		defaultValue = b;
		nbExpectedValues = 0;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "boolean handler (with " + defaultValue + ")"; 
	}

	@Override
	public void optionSet() {
		if (defaultValue != null) {
			addValue(defaultValue);
		}
	}

	@Override
	protected Object getValue(ArgParser argParser, String arg)
			throws ArgParseException {
		return Boolean.valueOf(arg); 
	}

}
