/**
 * 
 */
package bpiwowar.argparser.handlers;

import java.lang.reflect.Field;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import org.apache.log4j.Logger;

import bpiwowar.argparser.ArgParser;
import bpiwowar.argparser.ArgumentGroupActivator;
import bpiwowar.argparser.EnumValue;
import bpiwowar.argparser.StringScanException;
import bpiwowar.argparser.StringScanner;

/**
 * @author bpiwowar
 * @date 06/09/2007
 * 
 */
@HandlerInformation(value = Enum.class)
public class EnumHandler extends GenericObjectsHandler {
	TreeMap<String, Enum<?>> enums = new TreeMap<String, Enum<?>>();

	private Map<Enum<?>, String[]> activateGroups = new TreeMap<Enum<?>, String[]>();

	final static private Logger LOGGER = Logger.getLogger(EnumHandler.class);

	@SuppressWarnings("unchecked")
	public EnumHandler(Object object, Field field) {
		super(object, field);

		// handle a default value (if the field has an EnumValue annotation)
		EnumValue enumValue = field.getAnnotation(EnumValue.class);
		if (enumValue != null) {
			for (String key : enumValue.value())
				enums.put(key, (Enum<?>) get());
		}

		Class<Enum> c = (Class<Enum>) field.getType();
		for (Field f : c.getDeclaredFields()) {
			if (field.getType() == f.getType()) {
				enumValue = f.getAnnotation(EnumValue.class);
				Enum<?> x;
				try {
					boolean a = f.isAccessible();
					if (!a)
						f.setAccessible(true);
					x = (Enum) f.get(null);
					if (!a)
						f.setAccessible(false);
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
				if (enumValue == null)
					enums.put(x.name(), x);
				else
					for (String key : enumValue.value())
						enums.put(key, x);

				ArgumentGroupActivator activateGroups = f
						.getAnnotation(ArgumentGroupActivator.class);
				if (activateGroups != null) {
					this.activateGroups.put(x, activateGroups.value());
				}
			}
		}
	}

	public Set<String> getNames() {
		return enums.keySet();
	}

	@Override
	public String toString() {
		return "EnumHandler of " + field.getType();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see yrla.argparser.holders.Holder#scan(yrla.argparser.StringScanner)
	 */
	public Object process(ArgParser argParser, StringScanner scanner)
			throws StringScanException {
		final String s = scanner.scanString();
		setValue(argParser, s);
		return s;
	}

	public void setValue(ArgParser argParser, String value) {
		try {
			final Enum<?> v = enums.get(value);
			if (v == null && !enums.containsKey(value))
				throw new StringScanException("Enum value '" + value
						+ "' is not valid");
			field.set(object, v);

			if (activateGroups == null || v == null)
				return;

			String[] groups = this.activateGroups.get(v);
			if (groups != null)
				for (String group : groups)
					argParser.activate(group);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}
