package bpiwowar.argparser.handlers;

import java.io.File;
import java.lang.reflect.Field;
import java.util.Collection;

import bpiwowar.argparser.ArgParser;
import bpiwowar.argparser.InvalidHolderException;
import bpiwowar.argparser.StringScanException;
import bpiwowar.argparser.StringScanner;
import bpiwowar.argparser.utils.ReadLineIterator;

/**
 * This handler reads the file and add each line as a string to the collection
 * 
 * @author B. Piwowarski <benjamin@bpiwowar.net>
 */
public class FileStringSetHandler extends GenericObjectsHandler {
	public FileStringSetHandler(Object object, Field field)
			throws InvalidHolderException {
		super(object, field);
		if (!Collection.class.isAssignableFrom(field.getType()))
			throw new InvalidHolderException("Field %s is not a collection",
					field);
		// TODO: check it implements Collection<? extends String>
		// Type type = field.getGenericType();
	}

	@Override
	protected Object process(ArgParser argParser, StringScanner scanner)
			throws StringScanException {
		try {
			File file = new File(scanner.scanString());
			Collection<String> set = (Collection<String>) field.get(object);

			if (set == null) {
				set = (Collection<String>) field.getType().newInstance();
				field.set(object, set);
			}
			for (String line : new ReadLineIterator(file))
				set.add(line);
			return set;
		} catch (RuntimeException e) {
			throw e;
		} catch (Exception e) {
			throw new StringScanException(e);
		}
	}
}