/**
 * 
 */
package bpiwowar.argparser.handlers;

import java.lang.annotation.Annotation;
import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;

import org.apache.log4j.Logger;

import bpiwowar.argparser.ArgParseException;
import bpiwowar.argparser.ArgParser;
import bpiwowar.argparser.IllegalArgumentValue;
import bpiwowar.argparser.StringScanException;
import bpiwowar.argparser.StringScanner;

/**
 * @author Benjamin Piwowarski <benjamin@bpiwowar.net>
 * @date 06/09/2007
 * 
 */
abstract public class GenericObjectsHandler extends GenericHandler {
	final static private Logger logger = Logger
			.getLogger(GenericObjectsHandler.class);

	/**
	 * Our current index (in case of an array) or a negative value if the
	 * underlying field is not an array
	 */
	int index = -1;

	protected int nbExpectedValues;

	private Constructor<?> constructor;

	/**
	 * Use {@link #GenericObjectsHandler(Object, Field, Class)}
	 * 
	 * @param object
	 * @param field
	 */
	@Deprecated
	public GenericObjectsHandler(Object object, Field field) {
		this(object, field, field.getType());
	}

	/**
	 * Construct a new wrapper
	 * 
	 * @param object
	 *            The object (either holder, or containing a holder field)
	 * @param field
	 *            The field of the object (or null if the holder is the object)
	 */
	public GenericObjectsHandler(Object object, Field field, Class<?> aClass) {
		super(object, field);
		if (field != null) {
			try {
				field.setAccessible(true);
				// Set the index to 0 if needed
				if (field.getType().isArray()) {
					index = 0;
				}
			} catch (Exception e) {
				throw new RuntimeException(e);
			}

			// Try to get the constructor
			try {
				constructor = aClass.getConstructor(new Class<?>[] {});
			} catch (NoSuchMethodException m) {

			}

		}

		// Computes the number of expected values
		if (field.getType().isArray())
			try {
				nbExpectedValues = Array.getLength(field.get(object));
			} catch (IllegalArgumentException e) {
				throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			}
		else
			nbExpectedValues = 1;
	}

	// FIXME: Clear this mess!

	@Override
	public final int parse(ArgParser argParser, String[] args, int idx)
			throws ArgParseException, StringScanException, IllegalArgumentValue {
		optionSet();
		for (int i = 0; i < nbExpectedValues; i++) {
			if (idx >= args.length)
				throw new ArgParseException(
						"All the arguments have been processed, but we still wanted to parse %d (field %s)",
						nbExpectedValues - i, field);
			process(argParser, new StringScanner(args[idx]));
		}

		return idx + nbExpectedValues;
	}

	protected Object process(ArgParser argParser, String value)
			throws StringScanException, ArgParseException, IllegalArgumentValue {
		return process(argParser, new StringScanner(value));
	}

	protected Object process(ArgParser argParser, StringScanner scanner)
			throws StringScanException, ArgParseException, IllegalArgumentValue {
		final Object value = getValue(argParser, scanner.getString());
		logger.debug(String.format(
				"Handler %s returned a value of type %s (%s)", getClass(),
				value != null ? value.getClass() : null, value));
		addValue(value);
		return value;
	}

	protected Object getValue(ArgParser argParser, String arg)
			throws ArgParseException {
		throw new RuntimeException("NOT IMPLEMENTED");
	}

	public void optionSet() {
	};

	/**
	 * Add a new value
	 * 
	 * @param value
	 *            The value
	 */
	protected void addValue(Object value) {
		try {
			if (index >= 0)
				Array.set(field.get(object), index++, value);
			else
				field.set(object, value);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Get the value of the object
	 * 
	 * @return
	 */
	protected Object get() {
		try {
			return field.get(object);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Creates a new instance of the field (if it makes sense)
	 * 
	 * @return A freshly created object
	 */
	protected Object newInstance() {
		Object o;
		try {
			o = constructor.newInstance();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		addValue(o);
		return o;

	}

	protected <T extends Annotation> T getAnnotation(Class<T> aClass) {
		return field == null ? object.getClass().getAnnotation(aClass) : field
				.getAnnotation(aClass);

	}
}
