package bpiwowar.argparser.handlers;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;

import bpiwowar.argparser.ArgParser;
import bpiwowar.argparser.InvalidHolderException;
import bpiwowar.argparser.StringScanException;
import bpiwowar.argparser.StringScanner;

/**
 * This handler uses the string constructor of the target class
 * 
 * @author Benjamin Piwowarski <benjamin@bpiwowar.net>
 */
public class StringConstructorHandler extends GenericObjectsHandler {
	private Constructor<? extends Object> constructor;

	public StringConstructorHandler(Object object, Field field, Class<?> aClass)
			throws InvalidHolderException {
		super(object, field, aClass);
		final Class<?> type = field.getType();
		try {
			constructor = type.getConstructor(String.class);
		} catch (SecurityException e) {
			throw new InvalidHolderException(
					"Cannot get the String constructor for object %s", e, type);
		} catch (NoSuchMethodException e) {
			throw new InvalidHolderException(
					"Cannot get the String constructor for object %s", e, type);
		}
	}

	@Override
	protected Object process(ArgParser argParser, StringScanner scanner)
			throws StringScanException {
		String value = scanner.scanString();
		try {
			Object object = constructor.newInstance(value);
			addValue(object);
			return object;
		} catch (RuntimeException e) {
			throw e;
		} catch (Throwable e) {
			throw new StringScanException(e.getMessage());
		}
	}

}
