package bpiwowar.argparser.handlers;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.reflect.Field;

import org.apache.log4j.Logger;

import bpiwowar.argparser.ArgParser;
import bpiwowar.argparser.StringScanException;
import bpiwowar.argparser.StringScanner;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.converters.Converter;

/**
 * Handler for XStream serialized objects
 * 
 * @author Benjamin Piwowarski <benjamin@bpiwowar.net>
 * 
 */
public class XStreamHandler extends GenericObjectsHandler {
	private static final Logger logger = Logger.getLogger(XStreamHandler.class);

	@Retention(RetentionPolicy.RUNTIME)
	public @interface Options {
		Class<? extends Converter>[] converters() default {};
	}

	public XStreamHandler(Object object, Field field) {
		super(object, field);
	}

	@Override
	protected Object process(ArgParser argParser, StringScanner scanner)
			throws StringScanException {
		String name = scanner.getString();
		File file = new File(name);

		final XStream xstream = new XStream();
//		xstream.processAnnotations(object.getClass());
		xstream.autodetectAnnotations(true);

		for (Options options : new Options[] {
				field.getAnnotation(Options.class),
				field.getClass().getAnnotation(Options.class) })
			if (options != null) {
				for (Class<? extends Converter> converter : options
						.converters()) {
					logger.info("Adding XStream converter " + converter);
					try {
						xstream.registerConverter(converter.newInstance());
						logger.debug("Added converter " + converter);
					} catch (InstantiationException e) {
						logger.error("Cannot add the converter " + e);
					} catch (IllegalAccessException e) {
						logger.error("Cannot add the converter " + e);
					}
				}
			}

		if (file.isFile()) {
			try {
				Object object = xstream.fromXML(new FileInputStream(file));
				logger.debug(String.format("Loaded object %s from file %s",
						field, file));
				addValue(object);
				return object;
			} catch (FileNotFoundException e) {
				throw new StringScanException(e);
			}
		} else {
			// Try to parse XML
		}
		throw new RuntimeException("Not implemented");
	}
}
