package bpiwowar.argparser.holders;

import bpiwowar.argparser.ArgParseException;



/**
 * Wrapper class which ``holds'' a boolean value, enabling methods to return
 * boolean values through arguments.
 */
@SuppressWarnings("serial")
public class BooleanHolder extends Holder<Boolean> implements java.io.Serializable {
	/**
	 * Value of the boolean, set and examined by the application as needed.
	 */
	private boolean value;

	/**
	 * Constructs a new <code>BooleanHolder</code> with an initial value of
	 * <code>false</code>.
	 */
	public BooleanHolder() {
		setValue(false);
	}

	/**
	 * Constructs a new <code>BooleanHolder</code> with a specific initial
	 * value.
	 * 
	 * @param b
	 *            Initial boolean value.
	 */
	public BooleanHolder(boolean b) {
		setValue(b);
	}
	
	/**
	 * @param value
	 *            the value to set
	 */
	public void setValue(boolean value) {
		this.value = value;
	}

	/**
	 * @return the value
	 */
	public Boolean getValue() {
		return value;
	}

	@Override
	protected void setValue(String argument) throws ArgParseException {
		this.value = Boolean.parseBoolean(argument);
	}
}
