package bpiwowar.argparser.holders;

import bpiwowar.argparser.ArgParseException;


/**
 * Wrapper class which ``holds'' a character value, enabling methods to return
 * character values through arguments.
 */
public class CharHolder extends Holder<Character> implements java.io.Serializable {
	private static final long serialVersionUID = 1799709958161491946L;

	/**
	 * Value of the character, set and examined by the application as needed.
	 */
	public char value;

	/**
	 * Constructs a new <code>CharHolder</code> with an initial value of 0.
	 */
	public CharHolder() {
		value = 0;
	}

	/**
	 * Constructs a new <code>CharHolder</code> with a specific initial value.
	 * 
	 * @param c
	 *            Initial character value.
	 */
	public CharHolder(char c) {
		value = c;
	}
	
	@Override
	public Character getValue() {
		return value;
	}

	@Override
	protected void setValue(String argument) throws ArgParseException {
		if (argument.length() != 1)
			throw new ArgParseException("\"%s\" is not a character", argument);
		value = argument.charAt(0);
	}

}
