package bpiwowar.experiments;

import java.util.Arrays;

import bpiwowar.argparser.utils.Output;

public class PackageName implements Comparable<PackageName> {
	String[] packageName;

	/**
	 * Construction
	 */
	public PackageName(String[] packageName) {
		super();
		this.packageName = packageName;
	}

	public PackageName(PackageName key, int length) {
		packageName = new String[length];
		for(int i = 0; i < length; i++)
			packageName[i] = key.packageName[i];
	}

	@Override
	public String toString() {
		return Output.toString(".", packageName);
	}
	
	public int getLength() {
		return packageName.length;
	}
	
	/**
	 * Returns the length of the common prefix
	 */
	public int commonPrefixLength(PackageName o) {
		if (o == null)
			return 0;

		int min = Math.min(packageName.length, o.packageName.length);
		for (int i = 0; i < min; i++) {
			int z = packageName[i].compareTo(o.packageName[i]);
			if (z != 0)
				return i;
		}
		return min;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(packageName);
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		PackageName other = (PackageName) obj;
		if (!Arrays.equals(packageName, other.packageName)) {
			return false;
		}
		return true;
	}

	@Override
	public int compareTo(PackageName o) {
		// Compare package name first
		int min = Math.min(packageName.length, o.packageName.length);
		for (int i = 0; i < min; i++) {
			int z = packageName[i].compareTo(o.packageName[i]);
			if (z != 0)
				return z;
		}

		// The longer one then?
		int z = packageName.length - o.packageName.length;
		return z;
	}

}
